/***************************************************************
 * Name:      eettrzbadialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2017
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "eettrzbadialog.h"
#include "utils/dxutils.h"
#include "dxdefs.h"

IMPLEMENT_DYNAMIC_CLASS(EettrzbaDialog, wxDialog)

BEGIN_EVENT_TABLE(EettrzbaDialog, wxDialog)
    EVT_BUTTON(ID_EETTRZBA_SEND, EettrzbaDialog::OnSend)
    EVT_BUTTON(ID_EETTRZBA_LATER, EettrzbaDialog::OnLater)
    EVT_BUTTON(wxID_CANCEL, EettrzbaDialog::OnCancel)
END_EVENT_TABLE()

EettrzbaDialog::EettrzbaDialog(wxWindow *parent, const EetData &data, const wxString &pokladna, int fakturaid, int zavpohid, bool platce, const wxString &dphdruh)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Evidence tržby")), m_data(data), m_pokladna(pokladna), m_fakturaid(fakturaid), m_zavpohid(zavpohid),
      m_platce(platce), m_dphdruh(dphdruh), m_cestSluz(nullptr), m_pouzitZboz1(nullptr), m_pouzitZboz2(nullptr), m_pouzitZboz3(nullptr)
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void EettrzbaDialog::createControls()
{
    int i=0;

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Celkem"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_celkem = new wxStaticText(this, wxID_ANY, dxutils::fromStdString(m_data.getCelkTrzba()));
    gridSizer->Add(m_celkem, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    if(m_platce)
    {
        if(!m_data.getZaklNepodlDph().empty())
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková částka plnění osvobozených od DPH")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_zaklNepodlDph = new wxStaticText(this, wxID_ANY, dxutils::fromStdString(m_data.getZaklNepodlDph()));
            gridSizer->Add(m_zaklNepodlDph, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
            i++;
        }

        if(!m_data.getZaklDan1().empty())
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celkový základ daně se základní sazbou DPH")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_zaklDan1 = new wxStaticText(this, wxID_ANY, dxutils::fromStdString(m_data.getZaklDan1()));
            gridSizer->Add(m_zaklDan1, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
            i++;
        }

        if(!m_data.getDan1().empty())
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková DPH se základní sazbou")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_dan1 = new wxStaticText(this, wxID_ANY, dxutils::fromStdString(m_data.getDan1()));
            gridSizer->Add(m_dan1, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
            i++;
        }

        if(!m_data.getZaklDan2().empty())
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celkový základ daně s první sníženou sazbou DPH")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_zaklDan2 = new wxStaticText(this, wxID_ANY, dxutils::fromStdString(m_data.getZaklDan2()));
            gridSizer->Add(m_zaklDan2, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
            i++;
        }

        if(!m_data.getDan2().empty())
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková DPH s první sníženou sazbou")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_dan2 = new wxStaticText(this, wxID_ANY, dxutils::fromStdString(m_data.getDan2()));
            gridSizer->Add(m_dan2, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
            i++;
        }

        if(!m_data.getZaklDan3().empty())
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celkový základ daně s druhou sníženou sazbou DPH")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_zaklDan3 = new wxStaticText(this, wxID_ANY, dxutils::fromStdString(m_data.getZaklDan3()));
            gridSizer->Add(m_zaklDan3, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
            i++;
        }

        if(!m_data.getDan3().empty())
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková DPH s druhou sníženou sazbou")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_dan3 = new wxStaticText(this, wxID_ANY, dxutils::fromStdString(m_data.getDan3()));
            gridSizer->Add(m_dan3, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
            i++;
        }

        if(m_dphdruh == "R1,2-C" && !m_data.getCestSluz().empty())
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková částka v režimu DPH pro cestovní službu")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_cestSluz = new wxSpinCtrlDouble(this, wxID_ANY);
            m_cestSluz->SetRange(0.0, dxutils::toCDouble(m_data.getCestSluz()));
            m_cestSluz->SetDigits(2);
            m_cestSluz->SetValue(dxutils::toCDouble(m_data.getCestSluz()));
            gridSizer->Add(m_cestSluz, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
            i++;
        }

        if(m_dphdruh == "R1,2-P")
        {
            if(!m_data.getPouzitZboz1().empty())
            {
                gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková částka v režimu DPH pro prodej použitého zboží se základní sazbou")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
                m_pouzitZboz1 = new wxSpinCtrlDouble(this, wxID_ANY);
                m_pouzitZboz1->SetRange(0.0, dxutils::toCDouble(m_data.getPouzitZboz1()));
                m_pouzitZboz1->SetDigits(2);
                m_pouzitZboz1->SetValue(dxutils::toCDouble(m_data.getPouzitZboz1()));
                gridSizer->Add(m_pouzitZboz1, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
                i++;
            }

            if(!m_data.getPouzitZboz2().empty())
            {
                gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková částka v režimu DPH pro prodej použitého zboží s první sníženou sazbou")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
                m_pouzitZboz2 = new wxSpinCtrlDouble(this, wxID_ANY);
                m_pouzitZboz2->SetRange(0.0, dxutils::toCDouble(m_data.getPouzitZboz2()));
                m_pouzitZboz2->SetDigits(2);
                m_pouzitZboz2->SetValue(dxutils::toCDouble(m_data.getPouzitZboz2()));
                gridSizer->Add(m_pouzitZboz2, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
                i++;
            }

            if(!m_data.getPouzitZboz3().empty())
            {
                gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková částka v režimu DPH pro prodej použitého zboží s druhou sníženou sazbou")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
                m_pouzitZboz3 = new wxSpinCtrlDouble(this, wxID_ANY);
                m_pouzitZboz3->SetRange(0.0, dxutils::toCDouble(m_data.getPouzitZboz3()));
                m_pouzitZboz3->SetDigits(2);
                m_pouzitZboz3->SetValue(dxutils::toCDouble(m_data.getPouzitZboz3()));
                gridSizer->Add(m_pouzitZboz3, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
                i++;
            }
        }
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková částka plateb určená k následnému čerpání nebo zúčtování")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_urcenoCerpZuct = new wxSpinCtrlDouble(this, wxID_ANY);
    m_urcenoCerpZuct->SetRange(0.0, dxutils::toCDouble(m_data.getCelkTrzba()));
    m_urcenoCerpZuct->SetDigits(2);
    gridSizer->Add(m_urcenoCerpZuct, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková částka plateb, které jsou následným čerpáním nebo zúčtováním platby")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cerpZuct = new wxSpinCtrlDouble(this, wxID_ANY);
    m_cerpZuct->SetRange(0.0, dxutils::toCDouble(m_data.getCelkTrzba()));
    m_cerpZuct->SetDigits(2);
    gridSizer->Add(m_cerpZuct, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxBoxSizer *btnSizer = new wxBoxSizer(wxHORIZONTAL);
    wxButton* okButton = new wxButton(this, ID_EETTRZBA_SEND, wxString::FromUTF8("Odeslat teď"), wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->Add(okButton);
    wxButton* laterButton = new wxButton(this, ID_EETTRZBA_LATER, wxString::FromUTF8("Odeslat později"), wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->Add(laterButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->Add(cancelButton);
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void EettrzbaDialog::OnSend(wxCommandEvent& /*event*/)
{
    if(m_cestSluz) m_data.setCestSluz(m_cestSluz->GetValue());
    if(m_pouzitZboz1) m_data.setPouzitZboz1(m_pouzitZboz1->GetValue());
    if(m_pouzitZboz2) m_data.setPouzitZboz2(m_pouzitZboz2->GetValue());
    if(m_pouzitZboz3) m_data.setPouzitZboz3(m_pouzitZboz3->GetValue());
    if(!dxutils::isSameDouble(m_urcenoCerpZuct->GetValue(),0.0)) m_data.setUrcenoCerpZuct(m_urcenoCerpZuct->GetValue());
    if(!dxutils::isSameDouble(m_cerpZuct->GetValue(),0.0)) m_data.setCerpZuct(m_cerpZuct->GetValue());
    EetResult result = idbmodel->sendTrzba(m_pokladna, m_data, m_fakturaid, m_zavpohid);
    if(result.result == EET_OK || result.result == EET_VAROVANI)
    {
        if(result.warning.IsEmpty()) wxMessageDialog(this, wxString::FromUTF8("Tržba úspěšně odeslána"), "Informace", wxOK|wxICON_INFORMATION).ShowModal();
        else
        {
            wxMessageDialog(this, wxString::FromUTF8("Tržba odeslána s varováním:\n")+result.warning, "Informace", wxOK|wxICON_WARNING).ShowModal();
        }
    }
    else
    {
        wxMessageDialog(this, wxString::FromUTF8("Tržbu se nepodařilo odeslat.\nChyba:\n")+result.error, "Informace", wxOK|wxICON_ERROR).ShowModal();
    }
    EndModal(ID_EETTRZBA_SEND);
}

void EettrzbaDialog::OnLater(wxCommandEvent &/*event*/)
{
    if(m_cestSluz) m_data.setCestSluz(m_cestSluz->GetValue());
    if(m_pouzitZboz1) m_data.setPouzitZboz1(m_pouzitZboz1->GetValue());
    if(m_pouzitZboz2) m_data.setPouzitZboz2(m_pouzitZboz2->GetValue());
    if(m_pouzitZboz3) m_data.setPouzitZboz3(m_pouzitZboz3->GetValue());
    if(!dxutils::isSameDouble(m_urcenoCerpZuct->GetValue(),0.0)) m_data.setUrcenoCerpZuct(m_urcenoCerpZuct->GetValue());
    if(!dxutils::isSameDouble(m_cerpZuct->GetValue(),0.0)) m_data.setCerpZuct(m_cerpZuct->GetValue());
    EetResult result = idbmodel->addTrzba(m_pokladna, m_data, m_fakturaid, m_zavpohid);
    if(result.result == EET_OK) wxMessageDialog(this, wxString::FromUTF8("Tržba uložena k pozdějšímu odeslání"), "Informace", wxOK|wxICON_INFORMATION).ShowModal();
    else wxMessageDialog(this, wxString::FromUTF8("Tržbu se nepodařilo uložit.\nChyba:\n")+result.error, "Informace", wxOK|wxICON_ERROR).ShowModal();
    EndModal(ID_EETTRZBA_LATER);
}

void EettrzbaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}
