/***************************************************************
 * Name:      eettrzbydialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2017
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "eettrzbydialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "resources/images/csv.xpm"
#include "resources/images/checked.xpm"
#include "resources/images/unchecked.xpm"
#include "gui/prints/eetuctenkaprint.h"

IMPLEMENT_DYNAMIC_CLASS(EettrzbyDialog, wxDialog)

BEGIN_EVENT_TABLE(EettrzbyDialog, wxDialog)
    EVT_BUTTON(wxID_OK, EettrzbyDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, EettrzbyDialog::OnCancel)
    EVT_MENU(ID_EETTRZBY_EXPORT, EettrzbyDialog::OnExport)
    EVT_CHECKBOX(ID_EETTRZBY_NEODESLANE, EettrzbyDialog::OnNeodeslane)
    EVT_RADIOBOX(ID_EETTRZBY_FROMTO, EettrzbyDialog::OnFromTo)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_EETTRZBY_LIST, EettrzbyDialog::OnItemRightClick)
    EVT_MENU(ID_EETTRZBY_SEND, EettrzbyDialog::OnSend)
    EVT_MENU(ID_EETTRZBY_PRINTDOKLAD, EettrzbyDialog::OnPrintTrzba)
END_EVENT_TABLE()

EettrzbyDialog::EettrzbyDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Evidované tržby"))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_neodeslane = new wxCheckBox(this, ID_EETTRZBY_NEODESLANE, wxString::FromUTF8("Neodeslané"));
    m_neodeslane->SetValue(true);
    m_mainSizer->Add(m_neodeslane, 0, wxALL|wxEXPAND, 5);
    wxArrayString choices;
    choices.Add("Dnes");
    choices.Add(wxString::FromUTF8("Tento měsíc"));
    choices.Add(wxString::FromUTF8("Všechny"));
    m_fromto = new wxRadioBox(this, ID_EETTRZBY_FROMTO, wxString::FromUTF8("Zobrazit záznamy"), wxDefaultPosition, wxDefaultSize, choices);
    m_mainSizer->Add(m_fromto, 0, wxALL|wxEXPAND, 5);
    m_bar = new wxToolBar(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTB_HORIZONTAL|wxTB_FLAT|wxTB_NODIVIDER);
    m_bar->AddTool(ID_EETTRZBY_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, wxString::FromUTF8("Exportovat seznam tržeb"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(checked_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(unchecked_xpm).Scale(16, 16));
    m_list = new wxListCtrl(this, ID_EETTRZBY_LIST, wxDefaultPosition, wxDefaultSize, wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText(wxString::FromUTF8("Datum tržby"));
    col0.SetWidth(150);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Doklad");
    col1.SetWidth(110);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText("Celkem");
    col2.SetWidth(110);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText(wxString::FromUTF8("Odeslána"));
    col3.SetWidth(90);
    m_list->InsertColumn(3, col3);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void EettrzbyDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void EettrzbyDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void EettrzbyDialog::OnExport(wxCommandEvent &/*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat tržby do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << wxString::FromUTF8("Datum tržby") << delim << "Doklad" << delim << wxString::FromUTF8("Celková částka tržby") << delim << wxString::FromUTF8("Odeslána") << delim << wxString::FromUTF8("První zaslání údajů o tržbě") << delim;
    text << dxutils::enquotes(wxString::FromUTF8("Celková částka plnění osvobozených od DPH, ostatních plnění"), delim) << delim;
    text << wxString::FromUTF8("Celkový základ daně se základní sazbou DPH") << delim;
    text << wxString::FromUTF8("Celková DPH se základní sazbou") << delim;
    text << wxString::FromUTF8("Celkový základ daně s první sníženou sazbou DPH") << delim;
    text << wxString::FromUTF8("Celková DPH s první sníženou sazbou") << delim;
    text << wxString::FromUTF8("Celkový základ daně s druhou sníženou sazbou DPH") << delim;
    text << wxString::FromUTF8("DPH s druhou sníženou sazbou") << delim;
    text << wxString::FromUTF8("Celková částka v režimu DPH pro cestovní službu") << delim;
    text << wxString::FromUTF8("Celková částka v režimu DPH pro prodej použitého zboží se základní sazbou") << delim;
    text << dxutils::enquotes(wxString::FromUTF8("Celková částka v režimu DPH pro prodej použitého zboží s první sníženou sazbou"), delim) << delim;
    text << wxString::FromUTF8("Celková částka v režimu DPH pro prodej použitého zboží s druhou sníženou sazbou") << delim;
    text << wxString::FromUTF8("Celková částka plateb určená k následnému čerpání nebo zúčtování") << delim;
    text << dxutils::enquotes(wxString::FromUTF8("Celková částka plateb, které jsou následným čerpáním nebo zúčtováním platby"), delim) << delim;
    text << endl;
    for(size_t i=0; i<m_trzby.GetCount(); i++)
    {
        text << m_trzby[i].datTrzby().Format("%x %X");
        text << delim;
        text << dxutils::enquotes(idbmodel->hasFaktura(m_trzby[i].fakturaid())?idbmodel->getFaktura(m_trzby[i].fakturaid()).cislo():idbmodel->getZavPoh(m_trzby[i].zavpohid()).cislo(), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", m_trzby[i].celkTrzba()), delim);
        text << delim;
        text << (m_trzby[i].odeslana() ? "Ano" : "Ne");
        text << delim;
        text << (m_trzby[i].prvniZaslani() == PRVNI ? "Ano" : "Ne");
        text << delim;
        text << dxutils::enquotes(m_trzby[i].zaklNepodlDph(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].zaklDan1(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].dan1(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].zaklDan2(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].dan2(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].zaklDan3(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].dan3(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].cestSluz(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].pouzitZboz1(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].pouzitZboz2(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].pouzitZboz3(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].urcenoCerpZuct(), delim) << delim;
        text << dxutils::enquotes(m_trzby[i].cerpZuct(), delim) << delim;
        text << endl;
    }
}

void EettrzbyDialog::OnNeodeslane(wxCommandEvent &/*event*/)
{
    fillList();
}

void EettrzbyDialog::OnFromTo(wxCommandEvent &/*event*/)
{
    fillList();
}

void EettrzbyDialog::OnItemRightClick(wxListEvent &event)
{
    if(event.GetIndex() != -1)
    {
        if(m_trzby[event.GetIndex()].odeslana() && m_trzby[event.GetIndex()].celkTrzba() <= 0.0) return;
        //m_trzby[event.GetIndex()].fakturaid() == m_trzby[event.GetIndex()].zavpohid() znamena uz zrusena trzba
        if(m_trzby[event.GetIndex()].odeslana() && m_trzby[event.GetIndex()].fakturaid() == m_trzby[event.GetIndex()].zavpohid()) return;
        wxMenu popup;
        popup.Append(ID_EETTRZBY_SEND, m_trzby[event.GetIndex()].odeslana()?wxString::FromUTF8("Zrušit tržbu"):wxString::FromUTF8("Odeslat tržbu"));
        popup.AppendSeparator();
        popup.Append(ID_EETTRZBY_PRINTDOKLAD, wxString::FromUTF8("Vytisknout účtenku"));
        PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
    }
}

void EettrzbyDialog::OnSend(wxCommandEvent &/*event*/)
{
    long itemIndex = -1;
    while((itemIndex = m_list->GetNextItem(itemIndex,wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        break;
    }
    if(itemIndex != -1)
    {
        EetResult result = m_trzby[itemIndex].odeslana()?idbmodel->revokeTrzba(m_trzby[itemIndex].id()):idbmodel->resendTrzba(m_trzby[itemIndex].id());
        if(result.result == EET_OK || result.result == EET_VAROVANI)
        {
            if(result.result == EET_OK)
            {
                wxMessageDialog dialog(this, wxString::FromUTF8("Tržba odeslána"), wxString::FromUTF8("Výsledek odeslání"), wxOK|wxICON_INFORMATION);
                dialog.ShowModal();
            }
            if(result.result == EET_VAROVANI)
            {
                wxString msg = wxString::FromUTF8("Tržba odeslána\nVarování:\n");
                msg.Append(result.warning);
                wxMessageDialog dialog(this, msg, wxString::FromUTF8("Výsledek odeslání"), wxOK|wxICON_WARNING);
                dialog.ShowModal();
            }
            fillList();
        }
        else
        {
            wxString msg = wxString::FromUTF8("Tržba neodeslána\nChyba:\n");
            msg.Append(result.error);
            wxMessageDialog dialog(this, msg, wxString::FromUTF8("Výsledek odeslání"), wxOK|wxICON_ERROR);
            dialog.ShowModal();
        }
    }
}

void EettrzbyDialog::OnPrintTrzba(wxCommandEvent &/*event*/)
{
    long itemIndex = -1;
    while((itemIndex = m_list->GetNextItem(itemIndex,wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        break;
    }
    if(itemIndex != -1)
    {
        EetUctenkaPrint print(this, m_trzby[itemIndex]);
        //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
        print.Print();
#else
        print.Preview();
#endif
    }
}

void EettrzbyDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    wxDateTime from,to;
    if(m_fromto->GetSelection()==0) //dnes
    {
        from = wxDateTime::Today();
        to = wxDateTime(from.GetDay(), from.GetMonth(), from.GetYear(), 23, 59, 59);
    }
    else if(m_fromto->GetSelection()==1) //tento mesic
    {
        from = wxDateTime::Today();
        from.SetDay(1);
        to = from;
        to.SetToLastMonthDay();
        to.SetHour(23);
        to.SetMinute(59);
        to.SetSecond(59);
    }
    else
    {
        from = wxDateTime(1, wxDateTime::Jan, idbmodel->currentRok());
        to = wxDateTime(31, wxDateTime::Dec, idbmodel->currentRok(), 23, 59, 59);
    }
    m_trzby = idbmodel->eettrzby(from, to, m_neodeslane->GetValue());
    idbmodel->sortEettrzby(m_trzby);
    for(size_t i=0; i < m_trzby.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_trzby[i].datTrzby().Format("%x %X"));
        item.SetImage(-1);
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, idbmodel->hasFaktura(m_trzby[i].fakturaid())?idbmodel->getFaktura(m_trzby[i].fakturaid()).cislo():idbmodel->getZavPoh(m_trzby[i].zavpohid()).cislo());
        m_list->SetItem(static_cast<long>(i), 2, wxString::Format("%.2f", m_trzby[i].celkTrzba()));
        m_list->SetItem(static_cast<long>(i), 3, wxEmptyString, m_trzby[i].odeslana()?0:1);
    }
    m_list->Show();
    if(m_trzby.GetCount())
    {
        m_bar->EnableTool(ID_EETTRZBY_EXPORT, true);
    }
    else
    {
        m_bar->EnableTool(ID_EETTRZBY_EXPORT, false);
    }
}
