/***************************************************************
 * Name:      predvolbydialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/filename.h>
#include "predvolbydialog.h"
#include "radadialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "config.h"
#include "resources/images/rady32.xpm"
#include "resources/images/seznam32.xpm"
#include "resources/images/email32.xpm"
#include "resources/images/eet32.xpm"
#include "eet/eet.h"
#ifdef HAVE_CURL
    #include <curl/curl.h>
#endif

IMPLEMENT_DYNAMIC_CLASS(RadyPanel, wxPanel)

BEGIN_EVENT_TABLE(RadyPanel, wxPanel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PREDVOLBY_RADY_LIST, RadyPanel::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PREDVOLBY_RADY_LIST, RadyPanel::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PREDVOLBY_RADY_LIST, RadyPanel::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PREDVOLBY_RADY_LIST, RadyPanel::OnItemDeselected)
    EVT_MENU(ID_PREDVOLBY_RADY_EDIT, RadyPanel::OnEditRada)
END_EVENT_TABLE()

RadyPanel::RadyPanel(wxWindow *parent)
    : wxPanel(parent), m_selectedId(wxEmptyString)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_list = new wxListCtrl(this, ID_PREDVOLBY_RADY_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText("Popis");
    col0.SetWidth(250);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Prefix");
    col1.SetWidth(50);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("Znaků"));
    col2.SetWidth(50);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText(wxString::FromUTF8("Aktuální číslo"));
    col3.SetWidth(115);
    m_list->InsertColumn(3, col3);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void RadyPanel::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    if(!m_selectedId.IsEmpty())
    {
        wxMenu popup;
        popup.Append(ID_PREDVOLBY_RADY_EDIT, wxString::FromUTF8("Upravit řadu"));
        PopupMenu(&popup, event.GetPoint());
    }
}

void RadyPanel::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    RadaDialog dialog(this, m_selectedId);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void RadyPanel::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void RadyPanel::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void RadyPanel::OnEditRada(wxCommandEvent& /*event*/)
{
    if(m_selectedId.IsEmpty()) return;
    RadaDialog dialog(this, m_selectedId);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void RadyPanel::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = wxEmptyString;
    else
        m_selectedId = m_rady[selectedIndex].id();
}

void RadyPanel::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    m_rady = idbmodel->rady();
    for(size_t i=0; i < m_rady.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_rady[i].popis());
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_rady[i].prefix());
        m_list->SetItem(static_cast<long>(i), 2, wxString::Format("%d", m_rady[i].znaku()));
        m_list->SetItem(static_cast<long>(i), 3, wxString::Format("%d", m_rady[i].cislo()));
    }
    m_list->Show();
    setSelectedId(-1);
}

wxString RadyPage::GetName() const
{
    return wxString::FromUTF8("Číselné řady");
}

wxBitmap RadyPage::GetLargeIcon() const
{
    return wxBitmap(rady32_xpm);
}

wxWindow *RadyPage::CreateWindow(wxWindow *parent)
{
    return new RadyPanel(parent);
}

FakturacePanel::FakturacePanel(wxWindow *parent)
    : wxPanel(parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Splatnost"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_splatnost = new wxSpinCtrl(this);
    m_splatnost->SetRange(0,1000);
    gridSizer->Add(m_splatnost, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    if(wxPreferencesEditor::ShouldApplyChangesImmediately())
    {
        m_splatnost->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(FakturacePanel::SaveSplatnost), nullptr, this);
    }
}

bool FakturacePanel::TransferDataToWindow()
{
    m_splatnost->SetValue(dxsettings.splatnost());
    return true;
}

bool FakturacePanel::TransferDataFromWindow()
{
    wxSpinEvent dummy;
    SaveSplatnost(dummy);
    return true;
}

void FakturacePanel::SaveSplatnost(wxSpinEvent &/*event*/)
{
    dxsettings.setSplatnost(m_splatnost->GetValue());
    idbmodel->editSettings(Settings(idbmodel->currentRok(), m_splatnost->GetValue()));
}

wxString FakturacePage::GetName() const
{
    return "Fakturace";
}

wxBitmap FakturacePage::GetLargeIcon() const
{
    return wxBitmap(seznam32_xpm);
}

wxWindow *FakturacePage::CreateWindow(wxWindow *parent)
{
    return new FakturacePanel(parent);
}


wxString EmailPage::GetName() const
{
    return "E-mail";
}

wxBitmap EmailPage::GetLargeIcon() const
{
    return wxBitmap(email32_xpm);
}

wxWindow *EmailPage::CreateWindow(wxWindow *parent)
{
    return new EmailPanel(parent);
}


EmailPanel::EmailPanel(wxWindow *parent)
    : wxPanel(parent), m_useSsl(false)
{
    int i = 0;
#ifdef CURL_VERSION_SSL
    m_useSsl = true;
#endif

    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "SMTP server"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_server = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_server, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Port"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_port = new wxSpinCtrl(this);
    m_port->SetRange(0,65535);
    gridSizer->Add(m_port, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    if(m_useSsl)
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, "SSL"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_ssl = new wxChoice(this, wxID_ANY);
        m_ssl->Append(wxString::FromUTF8("Žádné"));
        m_ssl->Append("SSL/TLS");
        gridSizer->Add(m_ssl, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        i++;
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Použít autentifikaci")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_auth = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_auth, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Uživatelské jméno")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_username = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_username, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Heslo"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_password = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_password, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Odesílatel")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_sender = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_sender, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Předmět")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_subject = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_subject, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Text"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_text = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(300,70), wxTE_MULTILINE);
    gridSizer->Add(m_text, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    if(wxPreferencesEditor::ShouldApplyChangesImmediately())
    {
        m_server->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_port->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(EmailPanel::OnSpinEvent), nullptr, this);
        if(m_useSsl)
        {
            m_ssl->Connect(wxEVT_CHOICE, wxCommandEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        }
        m_auth->Connect(wxEVT_CHECKBOX, wxCommandEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_username->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_password->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_sender->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_subject->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_text->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
    }
}

bool EmailPanel::TransferDataToWindow()
{
    Smtp smtp = idbmodel->smtp();
    m_server->SetValue(smtp.server());
    m_port->SetValue(smtp.port());
    if(m_useSsl)
    {
        m_ssl->Select(smtp.ssl());
    }
    m_auth->SetValue(smtp.autentifikace());
    m_username->SetValue(smtp.username());
    m_password->SetValue(smtp.password());
    m_sender->SetValue(smtp.sender());
    m_subject->SetValue(smtp.subject());
    m_text->SetValue(smtp.text());

    return true;
}

bool EmailPanel::TransferDataFromWindow()
{
    if(m_useSsl)
    {
        idbmodel->setSmtp(Smtp(m_server->GetValue(), m_port->GetValue(), m_ssl->GetSelection(), m_auth->GetValue(), m_username->GetValue(),
                               m_password->GetValue(), m_sender->GetValue(), m_subject->GetValue(), m_text->GetValue()));
    }
    else
    {
        idbmodel->setSmtp(Smtp(m_server->GetValue(), m_port->GetValue(), 0, m_auth->GetValue(), m_username->GetValue(),
                               m_password->GetValue(), m_sender->GetValue(), m_subject->GetValue(), m_text->GetValue()));
    }
    return true;
}

void EmailPanel::OnCommandEvent(wxCommandEvent& /*event*/)
{
    TransferDataFromWindow();
}

void EmailPanel::OnSpinEvent(wxSpinEvent& /*event*/)
{
    TransferDataFromWindow();
}

wxString EetPage::GetName() const
{
    return "EET";
}

wxBitmap EetPage::GetLargeIcon() const
{
    return wxBitmap(eet32_xpm);
}

wxWindow *EetPage::CreateWindow(wxWindow *parent)
{
    return new EetPanel(parent);
}

IMPLEMENT_DYNAMIC_CLASS(EetPanel, wxPanel)

BEGIN_EVENT_TABLE(EetPanel, wxPanel)
    EVT_BUTTON(ID_PREDVOLBY_EET_CERT, EetPanel::OnCertPath)
    EVT_BUTTON(ID_PREDVOLBY_EET_TEST, EetPanel::OnTest)
END_EVENT_TABLE()

EetPanel::EetPanel(wxWindow *parent)
    : wxPanel(parent)
{
    int i = 0;

    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Provozovna"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_provozovna = new wxSpinCtrl(this);
    m_provozovna->SetRange(1,999999);
    gridSizer->Add(m_provozovna, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Režim tržby")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_rezim = new wxChoice(this, wxID_ANY);
    m_rezim->Append(wxString::FromUTF8("Běžný"));
    m_rezim->Append(wxString::FromUTF8("Zjednodušený"));
    gridSizer->Add(m_rezim, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Cesta k certifikátu")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cert = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(200,-1), wxTE_READONLY);
    gridSizer->Add(m_cert, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_certPath = new wxButton(this, ID_PREDVOLBY_EET_CERT, "...");
    gridSizer->Add(m_certPath, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Heslo k certifikátu")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pass = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(275,-1), wxTE_PASSWORD);
    gridSizer->Add(m_pass, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Evidovat od"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_od = new wxDatePickerCtrl(this, wxID_ANY, wxDefaultDateTime, wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_od->SetRange(wxDateTime(1, wxDateTime::Dec, 2016), wxDateTime(31, wxDateTime::Dec, wxDateTime::Today().GetYear()));
    gridSizer->Add(m_od, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ pověřujícího poplatníka")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dic = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_dic, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL, 5);
#if defined (__WXMAC__)
#else
    m_test = new wxButton(this, ID_PREDVOLBY_EET_TEST, wxString::FromUTF8("Otestovat odeslání"));
    m_mainSizer->Add(m_test);
#endif
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    if(wxPreferencesEditor::ShouldApplyChangesImmediately())
    {
        m_cert->Connect(wxEVT_TEXT, wxTextEventHandler(EetPanel::OnCommandEvent), nullptr, this);
        m_provozovna->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(EetPanel::OnSpinEvent), nullptr, this);
        m_rezim->Connect(wxEVT_CHOICE, wxCommandEventHandler(EetPanel::OnCommandEvent), nullptr, this);
        m_od->Connect(wxEVT_DATE_CHANGED, wxDateEventHandler(EetPanel::OnDateEvent), nullptr, this);
        m_pass->Connect(wxEVT_TEXT, wxTextEventHandler(EetPanel::OnCommandEvent), nullptr, this);
        m_dic->Connect(wxEVT_TEXT, wxTextEventHandler(EetPanel::OnCommandEvent), nullptr, this);
    }
}

bool EetPanel::TransferDataToWindow()
{
    EetSettings es = idbmodel->eetsettings();
    m_provozovna->SetValue(es.provozovna());
    wxFileName fn(es.cert());
    m_cert->SetValue(fn.GetFullName());
    m_cert->SetToolTip(fn.GetFullPath());
    m_pass->SetValue(es.pass());
    m_rezim->SetSelection(es.rezim());
    m_od->SetValue(es.od());
    m_dic->SetValue(es.dicpoverujiciho());
    return true;
}

bool EetPanel::TransferDataFromWindow()
{
    idbmodel->setEetSettings(EetSettings(m_provozovna->GetValue(), m_cert->GetToolTipText(), m_pass->GetValue(), (REZIM)m_rezim->GetSelection(), m_od->GetValue(), m_dic->GetValue()));
    return true;
}

void EetPanel::OnCertPath(wxCommandEvent &/*event*/)
{
    wxFileName fn(m_cert->GetToolTipText());
    wxFileDialog dialog(this, wxString::FromUTF8("Otevřít certifikát"), fn.Exists()?fn.GetPath():wxGetHomeDir(), fn.Exists()?fn.GetFullName():"",
                        wxString::FromUTF8("Soubory certifikátu (*.p12)|*.p12"),
                        wxFD_OPEN|wxFD_FILE_MUST_EXIST);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxString path = dialog.GetPath();
        if(!path.empty())
        {
            m_cert->SetToolTip(path);
            m_cert->SetValue(dialog.GetFilename());
        }
    }
}

void EetPanel::OnCommandEvent(wxCommandEvent &/*event*/)
{
    TransferDataFromWindow();
}

void EetPanel::OnSpinEvent(wxSpinEvent &/*event*/)
{
    TransferDataFromWindow();
}

void EetPanel::OnDateEvent(wxDateEvent &/*event*/)
{
    TransferDataFromWindow();
}

void EetPanel::OnTest(wxCommandEvent &/*event*/)
{
    wxBeginBusyCursor();
    Eet eet(dxutils::toStdString(idbmodel->firma().dic()), m_provozovna->GetValue(), dxutils::toStdString(m_cert->GetToolTipText()), dxutils::toStdString(m_pass->GetValue()),
            "POK1");
    eet.setDicPoverujiciho(dxutils::toStdString(m_dic->GetValue()));
    eet.setRezim((REZIM)m_rezim->GetSelection());
    eet.setOvereni(OVEROVACI);
    eet.setPlayground(false);
    EETCODE result = eet.sendTrzba(EetData("1A-13", 113.0));
    if(result == EET_OVERENO)
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Datovou zprávu evidované tržby v ověřovacím módu se podařilo zpracovat"), wxString::FromUTF8("Výsledek testu"), wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
    }
    if(result == EET_VAROVANI)
    {
        wxString msg = wxString::FromUTF8("Datovou zprávu evidované tržby v ověřovacím módu se podařilo zpracovat\nVarování:\n");
        msg.Append(eet.getVarovani());
        wxMessageDialog dialog(this, msg, wxString::FromUTF8("Výsledek testu"), wxOK|wxICON_WARNING);
        dialog.ShowModal();
    }
    if(result == EET_CHYBA || result == EET_ERROR)
    {
        wxString msg = "Chyba:\n";
        msg.Append(eet.getChyba());
        wxMessageDialog dialog(this, msg, wxString::FromUTF8("Výsledek testu"), wxOK|wxICON_ERROR);
        dialog.ShowModal();
    }
    wxEndBusyCursor();
}
