/***************************************************************
 * Name:      predvolbydialog.h
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef PREDVOLBYDIALOG_H
#define PREDVOLBYDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/preferences.h>
#include <wx/listctrl.h>
#include <wx/spinctrl.h>
#include <wx/datectrl.h>
#include <wx/dateevt.h>
#include "model/dbmodel.h"

class RadyPanel: public wxPanel
{
    DECLARE_DYNAMIC_CLASS(RadyPanel)
    DECLARE_EVENT_TABLE()
public:
    RadyPanel() : m_mainSizer(nullptr), m_list(nullptr) {}
    explicit RadyPanel(wxWindow *parent);
private:
    wxBoxSizer *m_mainSizer;
    wxListCtrl *m_list;
    wxString m_selectedId;
    dxRadaArray m_rady;

    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnEditRada(wxCommandEvent& event);
    void setSelectedId(long selectedIndex);
    void fillList();
};

class RadyPage: public wxPreferencesPage
{
public:
    virtual wxString GetName() const;
    virtual wxBitmap GetLargeIcon() const;
    virtual wxWindow *CreateWindow(wxWindow *parent);
};

class FakturacePanel: public wxPanel
{
public:
    FakturacePanel() : m_mainSizer(nullptr), m_splatnost(nullptr) {}
    explicit FakturacePanel(wxWindow *parent);

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();
private:
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_splatnost;

    void SaveSplatnost(wxSpinEvent& event);
};

class FakturacePage: public wxPreferencesPage
{
public:
    virtual wxString GetName() const;
    virtual wxBitmap GetLargeIcon() const;
    virtual wxWindow *CreateWindow(wxWindow *parent);
};

class EmailPanel: public wxPanel
{
public:
    EmailPanel() : m_mainSizer(nullptr), m_server(nullptr), m_port(nullptr), m_ssl(nullptr), m_useSsl(false),
        m_auth(nullptr), m_username(nullptr), m_password(nullptr), m_sender(nullptr), m_subject(nullptr),
        m_text(nullptr) {}
    explicit EmailPanel(wxWindow *parent);

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();
private:
    wxBoxSizer *m_mainSizer;
    wxTextCtrl *m_server;
    wxSpinCtrl *m_port;
    wxChoice *m_ssl;
    bool m_useSsl;
    wxCheckBox *m_auth;
    wxTextCtrl *m_username;
    wxTextCtrl *m_password;
    wxTextCtrl *m_sender;
    wxTextCtrl *m_subject;
    wxTextCtrl *m_text;
    
    void OnCommandEvent(wxCommandEvent& event);
    void OnSpinEvent(wxSpinEvent& event);
};

class EmailPage: public wxPreferencesPage
{
public:
    virtual wxString GetName() const;
    virtual wxBitmap GetLargeIcon() const;
    virtual wxWindow *CreateWindow(wxWindow *parent);
};

class EetPanel: public wxPanel
{
    DECLARE_DYNAMIC_CLASS(EetPanel)
    DECLARE_EVENT_TABLE()
public:
    EetPanel() : m_mainSizer(nullptr), m_provozovna(nullptr), m_rezim(nullptr), m_cert(nullptr),
        m_pass(nullptr), m_dic(nullptr), m_od(nullptr), m_certPath(nullptr), m_test(nullptr) {}
    explicit EetPanel(wxWindow *parent);

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();
private:
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_provozovna;
    wxChoice *m_rezim;
    wxTextCtrl *m_cert;
    wxTextCtrl *m_pass;
    wxTextCtrl *m_dic;
    wxDatePickerCtrl *m_od;
    wxButton *m_certPath;
    wxButton *m_test;

    void OnCertPath(wxCommandEvent& event);
    void OnCommandEvent(wxCommandEvent& event);
    void OnSpinEvent(wxSpinEvent& event);
    void OnDateEvent(wxDateEvent& event);
    void OnTest(wxCommandEvent& event);
};

class EetPage: public wxPreferencesPage
{
public:
    virtual wxString GetName() const;
    virtual wxBitmap GetLargeIcon() const;
    virtual wxWindow *CreateWindow(wxWindow *parent);
};

#endif  /* PREDVOLBYDIALOG_H */

