/***************************************************************
 * Name:      eetuctenkaprint.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2017
 * License:   GPL3
 **************************************************************/

#include "eetuctenkaprint.h"
#include "utils/dxutils.h"

EetUctenkaPrint::EetUctenkaPrint(wxWindow *parent, const EetTrzba &trzba)
    : DocumentPrint(parent, wxString::FromUTF8("EET účtenka")), m_trzba(trzba)
{
    m_marginLeft = 10-PAGE_OFFSET;
    m_marginRight = 10+PAGE_OFFSET;
    m_marginTop = 10-PAGE_OFFSET;
    m_marginBottom = 10;
    createDocument();
}

void EetUctenkaPrint::createDocument()
{
    int height = 0;
    int page = 1;
    m_document = new Document;
    TextElement text = TextElement(1, wxString::FromUTF8("EET účtenka"), wxPoint(m_marginLeft,m_marginTop), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_CENTER,
                                   TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height),
                                       wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    firma.Add(wxString::Format(wxString::FromUTF8("DIČ: %s"), fi.dic()));
    firma.Add(wxString::Format("Telefon: %s", fi.telefon()));
    firma.Add(wxString::Format("E-mail: %s", fi.email()));
    firma.Add(wxString::Format("WWW: %s", fi.www()));
    firma.Add(wxString::Format("Registrace: %s", fi.registrace()));
    TextParagraphElement paragraph = TextParagraphElement(1, firma, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    height += paragraph.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));

    wxArrayString texty;
    wxArrayInt width;
    wxArrayInt alig;
    texty.Add(wxString::Format("Provozovna: %d", m_trzba.idProvoz()));
    texty.Add(wxString::Format("Pokladna: %s", m_trzba.idPokl()));
    width.Add((m_pageWidth-m_marginLeft-m_marginRight)/2);
    width.Add((m_pageWidth-m_marginLeft-m_marginRight)/2);
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_RIGHT);
    TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    texty.Clear();
    texty.Add(wxString::Format(wxString::FromUTF8("Číslo účtenky: %s"), m_trzba.poradCis()));
    texty.Add(wxString::Format(wxString::FromUTF8("Datum a čas: %s"), m_trzba.datTrzby().Format("%x %X")));
    row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    texty.Clear();
    texty.Add(wxString::Format(wxString::FromUTF8("Celková částka: %.2f"), m_trzba.celkTrzba()));
    texty.Add(m_trzba.rezim()==ZJEDNODUSENY?wxString::FromUTF8("Evidováno ve zjednodušeném režimu"):wxString::FromUTF8("Evidováno v běžném režimu"));
    row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    text = TextElement(page, wxString::Format("BKP: %s", m_trzba.bkp()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    wxString fik = "";
    if(!m_trzba.fik().IsEmpty()) fik = wxString::Format("FIK: %s", m_trzba.fik());
    else fik = wxString::Format("PKP: %s", m_trzba.pkp());
    text = TextElement(page, fik, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setForceSpace(true);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));

    m_document->setPages(1);
}
